"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.mapRootChanges = exports.nodeIdFromChangeAtom = void 0;
const index_js_1 = require("../core/index.js");
function nodeIdFromChangeAtom(changeAtom, fallbackRevision) {
    return (0, index_js_1.makeDetachedNodeId)(changeAtom.revision ?? fallbackRevision, changeAtom.localId);
}
exports.nodeIdFromChangeAtom = nodeIdFromChangeAtom;
/**
 * Converts a `Delta.Root` whose tree content is represented with by `TIn` instances
 * into a `Delta.Root`whose tree content is represented with by `TOut` instances.
 *
 * This function is useful for converting `Delta`s that represent tree content with cursors
 * into `Delta`s that represent tree content with a deep-comparable representation of the content.
 * See {@link assertDeltaEqual}.
 * @param root - The delta to convert. Not mutated.
 * @param func - The functions used to map tree content.
 */
function mapRootChanges(root, func) {
    const out = {};
    if (root.fields !== undefined) {
        out.fields = root.fields;
    }
    if (root.build !== undefined) {
        out.build = root.build.map(({ id, trees }) => ({
            id,
            trees: trees.map(func),
        }));
    }
    return out;
}
exports.mapRootChanges = mapRootChanges;
//# sourceMappingURL=deltaUtils.js.map