/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { IChannelStorageService } from "@fluidframework/datastore-definitions";
import { IGarbageCollectionData, ISummaryTreeWithStats, ITelemetryContext } from "@fluidframework/runtime-definitions";
import { IJsonCodec } from "../codec/index.js";
import { ChangeFamily, ChangeFamilyEditor, SchemaAndPolicy } from "../core/index.js";
import { JsonCompatibleReadOnly } from "../util/index.js";
import { EditManager, SummaryData } from "./editManager.js";
import { EditManagerEncodingContext } from "./editManagerCodecs.js";
import { Summarizable, SummaryElementParser, SummaryElementStringifier } from "./sharedTreeCore.js";
/**
 * Provides methods for summarizing and loading an `EditManager`
 */
export declare class EditManagerSummarizer<TChangeset> implements Summarizable {
    private readonly editManager;
    private readonly codec;
    private readonly schemaAndPolicy?;
    readonly key = "EditManager";
    constructor(editManager: EditManager<ChangeFamilyEditor, TChangeset, ChangeFamily<ChangeFamilyEditor, TChangeset>>, codec: IJsonCodec<SummaryData<TChangeset>, JsonCompatibleReadOnly, JsonCompatibleReadOnly, EditManagerEncodingContext>, schemaAndPolicy?: SchemaAndPolicy | undefined);
    getAttachSummary(stringify: SummaryElementStringifier, fullTree?: boolean, trackState?: boolean, telemetryContext?: ITelemetryContext): ISummaryTreeWithStats;
    summarize(stringify: SummaryElementStringifier, fullTree?: boolean, trackState?: boolean, telemetryContext?: ITelemetryContext): Promise<ISummaryTreeWithStats>;
    private summarizeCore;
    getGCData(fullGC?: boolean): IGarbageCollectionData;
    load(services: IChannelStorageService, parse: SummaryElementParser): Promise<void>;
}
//# sourceMappingURL=editManagerSummarizer.d.ts.map